require 'digest/md5'
require 'rabbit_file_split/byte_format'

module RabbitFileSplit
  class Bytes
    include RabbitFileSplit::ByteFormat
    #  GNU utils split.c bytes_split interface
    #  bytes_split (uintmax_t n_bytes, char *buf, size_t bufsize, size_t initial_read, uintmax_t max_files)

    BUFFER_SIZE = 4096
    attr_accessor :file_path, :file_base_name, :split_file_num, :file_list, :file_parent_path, :original_file_size, :split_work_time

    @split_file_num = 0
    @file_list = Array.new

    # @param [String] file_path target file path
    # @return [Bytes] RabbitFIleSplit::Bytes instance
    def initialize(file_path)
      @file_path = file_path
      @file_base_name = File::basename(file_path)
      @file_parent_path = File::dirname(file_path)
      @original_file_size = File.size(file_path)
    end

    # @param [String] n_bytes base byte num or N + byte unit
    # @param [String] file_prefix file_prefix. default is original file name
    # @param [String] dest_file_path generate split file directory
    # @param [String] buffer_size file buffer size
    # @return void
    def split(n_bytes, file_prefix: @file_base_name, dest_file_path: @file_parent_path, buffer_size: BUFFER_SIZE)
      start_time = Time.now
      n_bytes = to_byte(n_bytes)
      total_readed_byte = 0
      unit_readed_byte = 0
      @split_file_num  = 1
      io = File.open(@file_path, 'rb')
      w_file_io = File.open(create_split_file_path(dest_file_path,file_prefix,@split_file_num), 'wb')
      while(true)
        buffer = io.read(buffer_size)
        #puts buffer_size
        total_readed_byte += buffer_size
        unit_readed_byte += buffer_size
        w_file_io.write(buffer)

        #check total readed
        if (total_readed_byte + buffer_size) > @original_file_size
          total_rest_byte = @original_file_size - total_readed_byte
          if ( (unit_readed_byte + total_rest_byte) > n_bytes )
            # n_bytes over. file change
            rest_byte = n_bytes - unit_readed_byte
            if rest_byte > 0
              rest_buffer = io.read(rest_byte)
              w_file_io.write(rest_buffer)
            end
            w_file_io.close
            @split_file_num +=1
            #file change
            w_file_io = File.open(create_split_file_path(dest_file_path,file_prefix,@split_file_num), 'wb')
          end
          # read rest byte
          if total_rest_byte > 0
            rest_buffer = io.read(total_rest_byte)
            w_file_io.write(rest_buffer)
          end

          end_time = Time.now
          @split_work_time = end_time - start_time
          w_file_io.close
          return @file_list
        end

        #check n_bytes
        if (unit_readed_byte + buffer_size) > n_bytes
          rest_byte = n_bytes - unit_readed_byte
          if rest_byte > 0
            rest_buffer = io.read(rest_byte)
            w_file_io.write(rest_buffer)
          end
          w_file_io.close
          @split_file_num +=1
          #file change
          unit_readed_byte = 0
          w_file_io = File.open(create_split_file_path(dest_file_path,file_prefix,@split_file_num), 'wb')
        end
      end
    end

    #@return [Integer] delete file num
    def delete_all
      File.delete(*@file_list)
    end

    private

    #@param [String] dest_file_path
    #@param [String] file_prefix
    #@param [Integer] num split file sequence number
    #@return [String] done split file path
    def create_split_file_path(dest_file_path,  file_prefix, num)
      file_path = File.join(dest_file_path,  file_prefix + '_' + sprintf('%04d', num))
      @file_list = [] if @file_list.nil?
      @file_list.push(file_path)
      puts file_path
      file_path
    end

  end
end